import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Modal, notification } from 'antd'

import MKEmitter from '@/utils/events.js'
import Api from '@/api'
import { getSearchForm } from '@/templates/zshare/formconfig'

import SearchForm from '@/templates/sharecomponent/searchcomponent/searchform'
import DragElement from './dragsearch'
import './index.scss'

const { confirm } = Modal

class SearchComponent extends Component {
  static propTpyes = {
    config: PropTypes.object,        // 配置信息
    updatesearch: PropTypes.func     // 更新
  }

  state = {
    searchlist: null,    // 搜索条件集
    sqlVerifing: false,  // sql验证中
    visible: false,      // 模态框控制
    card: null           // 编辑中元素
  }

  /**
   * @description 搜索条件初始化
   */
  UNSAFE_componentWillMount () {
    this.setState({
      searchlist: fromJS(this.props.config.search).toJS()
    })
  }

  componentDidMount () {
    MKEmitter.addListener('addSearch', this.addSearch)
  }

  /**
   * @description 监听到搜索条件复制时，触发搜索条件编辑
   */
  UNSAFE_componentWillReceiveProps (nextProps) {
    const { searchlist } = this.state

    if (!is(fromJS(nextProps.config.search), fromJS(this.props.config.search)) && !is(fromJS(nextProps.config.search), fromJS(searchlist))) {
      this.setState({searchlist: fromJS(nextProps.config.search).toJS()})
    }
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  /**
   * @description 组件销毁，清除state更新，清除快捷键设置
   */
  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('addSearch', this.addSearch)
  }

  addSearch = (cardId, element, type) => {
    if (cardId !== this.props.config.uuid) return

    const { searchlist } = this.state

    if (type === 'replace') {
      delete element.focus
      this.setState({
        searchlist: searchlist.map(cell => {
          if (cell.field && cell.field.toLowerCase() === element.field.toLowerCase()) {
            return element
          }
          return cell
        }),
      })
    } else {
      this.setState({searchlist: [...searchlist, element]})
    }
    this.handleSearch(element)
  }

  /**
   * @description 搜索条件顺序调整，或拖拽添加
   */
  handleList = (list, card) => {
    const { config } = this.props

    if (card) {
      this.setState({searchlist: list})
      this.handleSearch(card)
    } else {
      this.setState({searchlist: list}, ()=> {  
        this.props.updatesearch({...config, search: list})
      })
    }
  }

  /**
   * @description 搜索条件编辑，获取搜索条件表单信息
   */
  handleSearch = (card) => {
    const { config } = this.props
    const { searchlist } = this.state
    let linkableFields = []

    searchlist.forEach(item => {
      if (item.uuid === card.uuid) return
      if (!['select', 'link', 'checkcard'].includes(item.type)) return
      if (item.type === 'checkcard' && item.multiple === 'true') return
      
      linkableFields.push({
        value: item.field,
        text: item.label
      })
    })

    let columns = null
    if (config.columns && config.columns.length) {
      columns = config.columns.map(item => {
        return {key: item.uuid, text: item.field, value: item.field, label: item.label}
      })
    }

    this.setState({
      visible: true,
      card: card,
      formlist: getSearchForm(card, linkableFields, columns, 'header')
    })
  }

  /**
   * @description 取消保存，如果元素为新添元素，则从序列中删除
   */
  editModalCancel = () => {
    const { card } = this.state

    if (card.focus) {
      let searchlist = fromJS(this.state.searchlist).toJS()

      searchlist = searchlist.filter(item => item.uuid !== card.uuid)

      this.setState({
        card: null,
        searchlist: searchlist,
        visible: false
      })
    } else {
      this.setState({
        card: null,
        visible: false
      })
    }
  }

  /**
   * @description 搜索修改后提交保存
   * 1、去除系统默认搜索条件
   * 2、字段及提示文字重复校验
   * 3、更新下拉菜单可选集合
   * 4、下拉菜单数据源语法验证
   */
  handleSubmit = () => {
    const { config } = this.props
    let _searchlist = fromJS(this.state.searchlist).toJS()

    this.searchFormRef.handleConfirm().then(res => {
      let fieldrepet = false // 字段重复

      _searchlist = _searchlist.filter(item => !item.origin || item.uuid === res.uuid) // 去除系统项

      _searchlist = _searchlist.map(item => { // 数据更新及重复检测
        if (item.uuid !== res.uuid && res.field && item.field) {
          let itemFields = []
          if (item.type === 'text') {
            itemFields = item.field.split(',')
          } else if (item.type === 'group') {
            itemFields = [item.field, item.datefield]
          } else {
            itemFields = [item.field]
          }

          let resFields = []
          if (res.type === 'text') {
            resFields = res.field.split(',')
          } else if (res.type === 'group') {
            resFields = [res.field, res.datefield]
          } else {
            resFields = [res.field]
          }

          let setFields = Array.from(new Set([...itemFields, ...resFields]))

          if (setFields.length < itemFields.length + resFields.length && (res.type !== 'date' || item.type !== 'date')) {
            fieldrepet = true
          }
        }

        if (item.uuid === res.uuid) {
          return res
        } else {
          return item
        }
      })

      if (fieldrepet) {
        notification.warning({
          top: 92,
          message: '字段已存在!',
          duration: 5
        })
        return
      }

      if (['select', 'multiselect', 'link', 'checkcard'].includes(res.type) && res.resourceType === '1' && res.database !== 'sso' && /\s/.test(res.dataSource)) {
        this.setState({
          sqlVerifing: true
        })

        let sql = `declare @mk_departmentcode nvarchar(512),@mk_organization nvarchar(512),@mk_user_type nvarchar(20)
        ${res.dataSource}`
        sql = sql.replace(/@\$|\$@/ig, '')
        
        Api.sDebug(sql).then(result => {
          if (result.status || result.ErrCode === '-2') {
            this.setState({
              sqlVerifing: false,
              searchlist: _searchlist,
              visible: false
            }, ()=> {
              this.props.updatesearch({...config, search: _searchlist})
            })
          } else {
            this.setState({sqlVerifing: false})
            
            Modal.error({
              title: result.message
            })
          }
        })
      } else {
        this.setState({
          searchlist: _searchlist,
          visible: false
        }, ()=> { 
          this.props.updatesearch({...config, search: _searchlist})
        })
      }
    })
  }

  /**
   * @description 搜索条件删除
   */
  deleteElement = (card) => {
    const { config } = this.props
    let that = this

    confirm({
      content: `确定删除 - ${card.label} ？`,
      onOk() {
        let _searchlist = fromJS(that.state.searchlist).toJS()

        _searchlist = _searchlist.filter(item => item.uuid !== card.uuid)

        that.setState({
          searchlist: _searchlist
        }, () => {
          that.props.updatesearch({...config, search: _searchlist})
        })
      },
      onCancel() {}
    })
  }

  render() {
    const { searchlist, visible, sqlVerifing } = this.state

    return (
      <div className={'model-custom-header-search-list search-length' + (searchlist.length)}>
        <DragElement
          list={searchlist}
          handleList={this.handleList}
          handleMenu={this.handleSearch}
          deleteMenu={this.deleteElement}
        />
        {/* 编辑搜索条件 */}
        <Modal
          title="搜索条件-编辑"
          visible={visible}
          width={950}
          maskClosable={false}
          onOk={this.handleSubmit}
          confirmLoading={sqlVerifing}
          onCancel={this.editModalCancel}
          destroyOnClose
        >
          <SearchForm
            card={this.state.card}
            formlist={this.state.formlist}
            inputSubmit={this.handleSubmit}
            wrappedComponentRef={(inst) => this.searchFormRef = inst}
          />
        </Modal>
      </div>
    )
  }
}

export default SearchComponent